@@ -2,24 +2,24 @@
 
 import copy
 import importlib
+import importlib.util
 import logging
 import os.path
 import pkgutil
 import sys
+from collections.abc import Iterable, Iterator
 from functools import reduce
 from typing import (
+    TYPE_CHECKING,
+    Any,
     Callable,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
     Optional,
-    Set,
-    Tuple,
+    Union,
     cast,
 )
 
 import jinja2.nodes
+import jinja2.parser
 from jinja2 import (
     Environment,
     FileSystemLoader,
@@ -33,6 +33,7 @@ from jinja2.sandbox import SandboxedEnvironment
 
 from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.errors import SQLFluffUserError, SQLTemplaterError
+from sqlfluff.core.formatter import FormatterInterface
 from sqlfluff.core.helpers.slice import is_zero_slice, slice_length
 from sqlfluff.core.templaters.base import (
     RawFileSlice,
@@ -40,9 +41,13 @@ from sqlfluff.core.templaters.base import (
     TemplatedFileSlice,
     large_file_check,
 )
+from sqlfluff.core.templaters.builtins.dbt import DBT_BUILTINS
 from sqlfluff.core.templaters.python import PythonTemplater
 from sqlfluff.core.templaters.slicers.tracer import JinjaAnalyzer, JinjaTrace
 
+if TYPE_CHECKING:  # pragma: no cover
+    from jinja2.runtime import Macro
+
 # Instantiate the templater logger
 templater_logger = logging.getLogger("sqlfluff.templater")
 
@@ -51,12 +56,11 @@ class UndefinedRecorder:
     """Similar to jinja2.StrictUndefined, but remembers, not fails."""
 
     # Tell Jinja this object is safe to call and does not alter data.
-    # https://jinja.palletsprojects.com/en/2.9.x/sandbox/#jinja2.sandbox.SandboxedEnvironment.is_safe_callable
-    unsafe_callable = False
     # https://jinja.palletsprojects.com/en/3.0.x/sandbox/#jinja2.sandbox.SandboxedEnvironment.is_safe_callable
+    unsafe_callable = False
     alters_data = False
 
-    def __init__(self, name: str, undefined_set: set) -> None:
+    def __init__(self, name: str, undefined_set: set[str]) -> None:
         self.name = name
         # Reference to undefined set to modify, it is assumed that the
         # calling code keeps a reference to this variable to they can
@@ -68,15 +72,25 @@ class UndefinedRecorder:
         self.undefined_set.add(self.name)
         return ""
 
-    def __getattr__(self, item) -> "UndefinedRecorder":
+    def __getattr__(self, item: str) -> "UndefinedRecorder":
         """Don't fail when called, remember instead."""
         self.undefined_set.add(self.name)
         return UndefinedRecorder(f"{self.name}.{item}", self.undefined_set)
 
-    def __call__(self, *args, **kwargs) -> "UndefinedRecorder":
+    def __getitem__(self, item: str) -> "UndefinedRecorder":
+        """Don't fail when called, remember instead."""
+        self.undefined_set.add(self.name)
+        return UndefinedRecorder(f"{self.name}.{item}", self.undefined_set)
+
+    def __call__(self, *args: Any, **kwargs: Any) -> "UndefinedRecorder":
         """Don't fail when called unlike parent class."""
         return UndefinedRecorder(f"{self.name}()", self.undefined_set)
 
+    def __iter__(self) -> Iterator["UndefinedRecorder"]:
+        """Don't fail when iterated, remember instead."""
+        self.undefined_set.add(self.name)
+        yield UndefinedRecorder(f"iter({self.name})", self.undefined_set)
+
 
 class JinjaTemplater(PythonTemplater):
     """A templater using the jinja2 library.
@@ -92,7 +106,9 @@ class JinjaTemplater(PythonTemplater):
         pass
 
     @staticmethod
-    def _extract_macros_from_template(template, env, ctx):
+    def _extract_macros_from_template(
+        template: str, env: Environment, ctx: dict[str, Any]
+    ) -> dict[str, "Macro"]:
         """Take a template string and extract any macros from it.
 
         Lovingly inspired by http://codyaray.com/2015/05/auto-load-jinja2-macros
@@ -105,7 +121,7 @@ class JinjaTemplater(PythonTemplater):
         from jinja2.runtime import Macro  # noqa
 
         # Iterate through keys exported from the loaded template string
-        context = {}
+        context: dict[str, Macro] = {}
         # NOTE: `env.from_string()` will raise TemplateSyntaxError if `template`
         # is invalid.
         macro_template = env.from_string(template, globals=ctx)
@@ -127,14 +143,19 @@ class JinjaTemplater(PythonTemplater):
 
     @classmethod
     def _extract_macros_from_path(
-        cls, path: List[str], env: Environment, ctx: Dict
-    ) -> dict:
+        cls,
+        path: list[str],
+        env: Environment,
+        ctx: dict[str, Any],
+        exclude_paths: Optional[list[str]] = None,
+    ) -> dict[str, "Macro"]:
         """Take a path and extract macros from it.
 
         Args:
-            path (List[str]): A list of paths.
+            path (list[str]): A list of paths.
             env (Environment): The environment object.
             ctx (Dict): The context dictionary.
+            exclude_paths (Optional[[List][str]]): A list of paths to exclude
 
         Returns:
             dict: A dictionary containing the extracted macros.
@@ -143,13 +164,18 @@ class JinjaTemplater(PythonTemplater):
             ValueError: If a path does not exist.
             SQLTemplaterError: If there is an error in the Jinja macro file.
         """
-        macro_ctx = {}
+        macro_ctx: dict[str, "Macro"] = {}
         for path_entry in path:
             # Does it exist? It should as this check was done on config load.
             if not os.path.exists(path_entry):
                 raise ValueError(f"Path does not exist: {path_entry}")
 
             if os.path.isfile(path_entry):
+                if exclude_paths:
+                    if cls._exclude_macros(
+                        macro_path=path_entry, exclude_macros_path=exclude_paths
+                    ):
+                        continue
                 # It's a file. Extract macros from it.
                 with open(path_entry) as opened_file:
                     template = opened_file.read()
@@ -172,12 +198,17 @@ class JinjaTemplater(PythonTemplater):
                         if fname.endswith(".sql"):
                             macro_ctx.update(
                                 cls._extract_macros_from_path(
-                                    [os.path.join(dirpath, fname)], env=env, ctx=ctx
+                                    [os.path.join(dirpath, fname)],
+                                    env=env,
+                                    ctx=ctx,
+                                    exclude_paths=exclude_paths,
                                 )
                             )
         return macro_ctx
 
-    def _extract_macros_from_config(self, config, env, ctx):
+    def _extract_macros_from_config(
+        self, config: FluffConfig, env: Environment, ctx: dict[str, Any]
+    ) -> dict[str, "Macro"]:
         """Take a config and load any macros from it.
 
         Args:
@@ -189,7 +220,6 @@ class JinjaTemplater(PythonTemplater):
             dict: A dictionary containing the extracted macros.
         """
         if config:
-            # This is now a nested section
             loaded_context = (
                 config.get_section((self.templater_selector, self.name, "macros")) or {}
             )
@@ -197,7 +227,7 @@ class JinjaTemplater(PythonTemplater):
             loaded_context = {}
 
         # Iterate to load macros
-        macro_ctx = {}
+        macro_ctx: dict[str, "Macro"] = {}
         for value in loaded_context.values():
             try:
                 macro_ctx.update(
@@ -209,7 +239,7 @@ class JinjaTemplater(PythonTemplater):
                 )
         return macro_ctx
 
-    def _extract_libraries_from_config(self, config):
+    def _extract_libraries_from_config(self, config: FluffConfig) -> dict[str, Any]:
         """Extracts libraries from the given configuration.
 
         This function iterates over the modules in the library path and
@@ -249,9 +279,13 @@ class JinjaTemplater(PythonTemplater):
             # import_module is deprecated as of python 3.4. This follows roughly
             # the guidance of the python docs:
             # https://docs.python.org/3/library/importlib.html#approximating-importlib-import-module
-            spec = module_finder.find_spec(module_name)
+            spec = module_finder.find_spec(module_name, None)
+            assert (
+                spec
+            ), f"Module {module_name} failed to be found despite being listed."
             module = importlib.util.module_from_spec(spec)
             sys.modules[module_name] = module
+            assert spec.loader, f"Module {module_name} missing expected loader."
             spec.loader.exec_module(module)
 
             if "." in module_name:  # nested modules have `.` in module_name
@@ -277,42 +311,10 @@ class JinjaTemplater(PythonTemplater):
         # remove magic methods from result
         return {k: v for k, v in libraries.__dict__.items() if not k.startswith("__")}
 
-    @staticmethod
-    def _generate_dbt_builtins():
-        """Generate the dbt builtins which are injected in the context."""
-        # This feels a bit wrong defining these here, they should probably
-        # be configurable somewhere sensible. But for now they're not.
-        # TODO: Come up with a better solution.
-
-        class ThisEmulator:
-            """A class which emulates the `this` class from dbt."""
-
-            name = "this_model"
-            schema = "this_schema"
-            database = "this_database"
-
-            def __str__(self) -> str:  # pragma: no cover TODO?
-                return self.name
-
-        dbt_builtins = {
-            "ref": lambda *args: args[-1],
-            # In case of a cross project ref in dbt, model_ref is the second
-            # argument. Otherwise it is the only argument.
-            "source": lambda source_name, table: f"{source_name}_{table}",
-            "config": lambda **kwargs: "",
-            "var": lambda variable, default="": "item",
-            # `is_incremental()` renders as True, always in this case.
-            # TODO: This means we'll never parse other parts of the query,
-            # that are only reachable when `is_incremental()` returns False.
-            # We should try to find a solution to that. Perhaps forcing the file
-            # to be parsed TWICE if it uses this variable.
-            "is_incremental": lambda: True,
-            "this": ThisEmulator(),
-        }
-        return dbt_builtins
-
     @classmethod
-    def _crawl_tree(cls, tree, variable_names, raw) -> Iterator[SQLTemplaterError]:
+    def _crawl_tree(
+        cls, tree: jinja2.nodes.Node, variable_names: set[str], raw: str
+    ) -> Iterator[SQLTemplaterError]:
         """Crawl the tree looking for occurrences of the undeclared values."""
         # First iterate through children
         for elem in tree.iter_child_nodes():
@@ -332,7 +334,7 @@ class JinjaTemplater(PythonTemplater):
                 line_pos=pos,
             )
 
-    def _get_jinja_env(self, config=None):
+    def _get_jinja_env(self, config: Optional[FluffConfig] = None) -> Environment:
         """Get a properly configured jinja environment.
 
         This method returns a properly configured jinja environment. It
@@ -352,7 +354,8 @@ class JinjaTemplater(PythonTemplater):
         Returns:
             jinja2.Environment: A properly configured jinja environment.
         """
-        macros_path = self._get_macros_path(config)
+        loader: Optional[FileSystemLoader]
+        macros_path = self._get_macros_path(config, "load_macros_from_path")
         loader_search_path = self._get_loader_search_path(config)
         final_search_path = (loader_search_path or []) + (macros_path or [])
 
@@ -360,12 +363,12 @@ class JinjaTemplater(PythonTemplater):
         if ignore_templating:
 
             class SafeFileSystemLoader(FileSystemLoader):
-                def get_source(self, environment, name, *args, **kwargs):
+                def get_source(
+                    self, environment: Environment, name: str
+                ) -> tuple[str, str, Callable[..., Any]]:
                     try:
                         if not isinstance(name, DummyUndefined):
-                            return super().get_source(
-                                environment, name, *args, **kwargs
-                            )
+                            return super().get_source(environment, name)
                         raise TemplateNotFound(str(name))
                     except TemplateNotFound:
                         # When ignore=templating is set, treat missing files
@@ -380,7 +383,7 @@ class JinjaTemplater(PythonTemplater):
             loader = SafeFileSystemLoader(final_search_path or [])
         else:
             loader = FileSystemLoader(final_search_path) if final_search_path else None
-        extensions = ["jinja2.ext.do"]
+        extensions: list[Union[str, type[Extension]]] = ["jinja2.ext.do"]
         if self._apply_dbt_builtins(config):
             extensions.append(DBTTestExtension)
 
@@ -393,11 +396,13 @@ class JinjaTemplater(PythonTemplater):
             loader=loader,
         )
 
-    def _get_macros_path(self, config: FluffConfig) -> Optional[List[str]]:
+    def _get_macros_path(
+        self, config: Optional[FluffConfig], key: str
+    ) -> Optional[list[str]]:
         """Get the list of macros paths from the provided config object.
 
         This method searches for a config section specified by the
-        templater_selector, name, and 'load_macros_from_path' keys. If the section is
+        templater_selector, name, and key specified. If the section is
         found, it retrieves the value associated with that section and splits it into
         a list of strings using a comma as the delimiter. The resulting list is
         stripped of whitespace and empty strings and returned. If the section is not
@@ -406,21 +411,23 @@ class JinjaTemplater(PythonTemplater):
         Args:
             config (FluffConfig): The config object to search for the macros path
                 section.
+            key (str): Key to load the macros path from the config file.
+                Also used for loading the excluding macros path from config.
 
         Returns:
-            Optional[List[str]]: The list of macros paths if found, None otherwise.
+            Optional[list[str]]: The list of macros paths if found, None otherwise.
         """
         if config:
-            macros_path = config.get_section(
-                (self.templater_selector, self.name, "load_macros_from_path")
-            )
+            macros_path = config.get_section((self.templater_selector, self.name, key))
             if macros_path:
                 result = [s.strip() for s in macros_path.split(",") if s.strip()]
                 if result:
                     return result
         return None
 
-    def _get_loader_search_path(self, config: FluffConfig) -> Optional[List[str]]:
+    def _get_loader_search_path(
+        self, config: Optional[FluffConfig]
+    ) -> Optional[list[str]]:
         """Get the list of Jinja loader search paths from the provided config object.
 
         This method searches for a config section specified by the
@@ -435,7 +442,7 @@ class JinjaTemplater(PythonTemplater):
                 path section.
 
         Returns:
-            Optional[List[str]]: The list of loader search paths if found, None
+            Optional[list[str]]: The list of loader search paths if found, None
                 otherwise.
         """
         if config:
@@ -456,7 +463,7 @@ class JinjaTemplater(PythonTemplater):
         """
         return JinjaAnalyzer(raw_str, env)
 
-    def _apply_dbt_builtins(self, config: FluffConfig) -> bool:
+    def _apply_dbt_builtins(self, config: Optional[FluffConfig]) -> bool:
         """Check if dbt builtins should be applied from the provided config object.
 
         This method searches for a config section specified by the
@@ -472,25 +479,42 @@ class JinjaTemplater(PythonTemplater):
             bool: True if dbt builtins should be applied, False otherwise.
         """
         if config:
-            return config.get_section(
+            apply_dbt_builtins = config.get_section(
                 (self.templater_selector, self.name, "apply_dbt_builtins")
             )
+            # If the config is totally absent for this templater, default to False,
+            # but for any other value that isn't boolean, throw an error.
+            if apply_dbt_builtins is None:
+                apply_dbt_builtins = False
+            assert isinstance(apply_dbt_builtins, bool), (
+                f"`apply_dbt_builtins` for {self.templater_selector}.{self.name} "
+                f"must be True/False, not {apply_dbt_builtins!r}"
+            )
+            return apply_dbt_builtins
         return False
 
-    def get_context(self, fname=None, config=None, **kw) -> Dict:
+    def _get_env_context(
+        self,
+        fname: Optional[str],
+        config: Optional[FluffConfig],
+        env: Environment,
+    ) -> dict[str, Any]:
         """Get the templating context from the config.
 
+        NOTE: This closely mirrors the `get_context` method which we inherit from the
+        python templater, but extends the signature. For that reason we define a new
+        method here, which internally refers to `get_context`.
+
         Args:
             fname (str, optional): The name of the file.
             config (dict, optional): The configuration.
-            **kw: Additional keyword arguments.
+            env: The Jinja Environment.
 
         Returns:
             dict: The templating context.
         """
         # Load the context
-        env = kw.pop("env")
-        live_context = super().get_context(fname=fname, config=config)
+        live_context = self.get_context(fname, config)
         # Apply dbt builtin functions if we're allowed.
         if config:
             # first make libraries available in the context
@@ -498,26 +522,29 @@ class JinjaTemplater(PythonTemplater):
             libraries = self._extract_libraries_from_config(config=config)
             live_context.update(libraries)
 
-            if libraries.get("SQLFLUFF_JINJA_FILTERS"):
-                env.filters.update(libraries.get("SQLFLUFF_JINJA_FILTERS"))
+            jinja_filters = libraries.get("SQLFLUFF_JINJA_FILTERS")
+            if jinja_filters:
+                env.filters.update(jinja_filters)
 
             if self._apply_dbt_builtins(config):
-                # This feels a bit wrong defining these here, they should probably
-                # be configurable somewhere sensible. But for now they're not.
-                # TODO: Come up with a better solution.
-                dbt_builtins = self._generate_dbt_builtins()
-                for name in dbt_builtins:
+                for name in DBT_BUILTINS:
                     # Only apply if it hasn't already been set at this stage.
                     if name not in live_context:
-                        live_context[name] = dbt_builtins[name]
+                        live_context[name] = DBT_BUILTINS[name]
 
         # Load macros from path (if applicable)
         if config:
-            macros_path = self._get_macros_path(config)
+            macros_path = self._get_macros_path(config, "load_macros_from_path")
+            exclude_macros_path = self._get_macros_path(
+                config, "exclude_macros_from_path"
+            )
             if macros_path:
                 live_context.update(
                     self._extract_macros_from_path(
-                        macros_path, env=env, ctx=live_context
+                        macros_path,
+                        env=env,
+                        ctx=live_context,
+                        exclude_paths=exclude_macros_path,
                     )
                 )
 
@@ -531,8 +558,8 @@ class JinjaTemplater(PythonTemplater):
         return live_context
 
     def construct_render_func(
-        self, fname=None, config=None
-    ) -> Tuple[Environment, dict, Callable[[str], str]]:
+        self, fname: Optional[str] = None, config: Optional[FluffConfig] = None
+    ) -> tuple[Environment, dict[str, Any], Callable[[str], str]]:
         """Builds and returns objects needed to create and run templates.
 
         Args:
@@ -540,7 +567,7 @@ class JinjaTemplater(PythonTemplater):
             config (Optional[dict]): The configuration settings.
 
         Returns:
-            Tuple[Environment, dict, Callable[[str], str]]: A tuple
+            tuple[Environment, dict, Callable[[str], str]]: A tuple
             containing the following:
                 - env (Environment): An instance of the 'Environment' class.
                 - live_context (dict): A dictionary containing the live context.
@@ -549,7 +576,7 @@ class JinjaTemplater(PythonTemplater):
         """
         # Load the context
         env = self._get_jinja_env(config)
-        live_context = self.get_context(fname=fname, config=config, env=env)
+        live_context = self._get_env_context(fname, config, env)
 
         def render_func(in_str: str) -> str:
             """Used by JinjaTracer to instantiate templates.
@@ -580,10 +607,10 @@ class JinjaTemplater(PythonTemplater):
         self,
         in_str: str,
         syntax_tree: jinja2.nodes.Template,
-        undefined_variables: Set[str],
-    ) -> List[SQLTemplaterError]:
+        undefined_variables: set[str],
+    ) -> list[SQLTemplaterError]:
         """Generates violations for any undefined variables."""
-        violations: List[SQLTemplaterError] = []
+        violations: list[SQLTemplaterError] = []
         if undefined_variables:
             # Lets go through and find out where they are:
             for template_err_val in self._crawl_tree(
@@ -594,17 +621,17 @@ class JinjaTemplater(PythonTemplater):
 
     @staticmethod
     def _init_undefined_tracking(
-        live_context: dict,
+        live_context: dict[str, Any],
         potentially_undefined_variables: Iterable[str],
         ignore_templating: bool = False,
-    ) -> Set[str]:
+    ) -> set[str]:
         """Sets up tracing of undefined template variables.
 
         NOTE: This works by mutating the `live_context` which
         is being used by the environment.
         """
         # NOTE: This set is modified by the `UndefinedRecorder` when run.
-        undefined_variables: Set[str] = set()
+        undefined_variables: set[str] = set()
 
         for val in potentially_undefined_variables:
             if val not in live_context:
@@ -622,8 +649,8 @@ class JinjaTemplater(PythonTemplater):
         in_str: str,
         fname: str,
         config: Optional[FluffConfig] = None,
-        formatter=None,
-    ) -> Tuple[TemplatedFile, List[SQLTemplaterError]]:
+        formatter: Optional[FormatterInterface] = None,
+    ) -> tuple[TemplatedFile, list[SQLTemplaterError]]:
         """Process a string and return the new string.
 
         Note that the arguments are enforced as keywords
@@ -648,7 +675,7 @@ class JinjaTemplater(PythonTemplater):
                 caught and displayed appropriately.
 
         Returns:
-            Tuple[TemplatedFile, List[SQLTemplaterError]]: A tuple containing the
+            tuple[TemplatedFile, list[SQLTemplaterError]]: A tuple containing the
             templated file and a list of violations.
         """
         if not config:  # pragma: no cover
@@ -711,7 +738,7 @@ class JinjaTemplater(PythonTemplater):
                 (
                     "Unrecoverable failure in Jinja templating: {}. Have you "
                     "correctly configured your variables? "
-                    "https://docs.sqlfluff.com/en/latest/configuration.html"
+                    "https://docs.sqlfluff.com/en/latest/perma/variables.html"
                 ).format(err),
                 # We don't have actual line number information, but specify
                 # line 1 so users can ignore with "noqa" if they want. (The
@@ -722,18 +749,22 @@ class JinjaTemplater(PythonTemplater):
             )
 
     def slice_file(
-        self, raw_str: str, render_func: Callable[[str], str], config=None, **kwargs
-    ) -> Tuple[List[RawFileSlice], List[TemplatedFileSlice], str]:
+        self,
+        raw_str: str,
+        render_func: Callable[[str], str],
+        config: Optional[FluffConfig] = None,
+        append_to_templated: str = "",
+    ) -> tuple[list[RawFileSlice], list[TemplatedFileSlice], str]:
         """Slice the file to determine regions where we can fix.
 
         Args:
             raw_str (str): The raw string to be sliced.
             render_func (Callable[[str], str]): The rendering function to be used.
             config (optional): Optional configuration.
-            **kwargs: Additional keyword arguments.
+            append_to_templated: Optional string to append to the template.
 
         Returns:
-            Tuple[List[RawFileSlice], List[TemplatedFileSlice], str]:
+            tuple[list[RawFileSlice], list[TemplatedFileSlice], str]:
                 A tuple containing a list of raw file slices, a list of
                 templated file slices, and the templated string.
         """
@@ -744,13 +775,13 @@ class JinjaTemplater(PythonTemplater):
         templater_logger.debug("    Raw String: %r", raw_str[:80])
         analyzer = self._get_jinja_analyzer(raw_str, self._get_jinja_env())
         tracer = analyzer.analyze(render_func)
-        trace = tracer.trace(append_to_templated=kwargs.pop("append_to_templated", ""))
+        trace = tracer.trace(append_to_templated=append_to_templated)
         return trace.raw_sliced, trace.sliced_file, trace.templated_str
 
     @staticmethod
     def _rectify_templated_slices(
-        length_deltas: Dict[int, int], sliced_template: List[TemplatedFileSlice]
-    ):
+        length_deltas: dict[int, int], sliced_template: list[TemplatedFileSlice]
+    ) -> list[TemplatedFileSlice]:
         """This method rectifies the source slices of a variant template.
 
         :TRICKY: We want to yield variants that _look like_ they were
@@ -770,7 +801,7 @@ class JinjaTemplater(PythonTemplater):
         # a dict and b) because they may have been generated out of order.
         delta_stack = sorted(length_deltas.items(), key=lambda t: t[0])
 
-        adjusted_slices: List[TemplatedFileSlice] = []
+        adjusted_slices: list[TemplatedFileSlice] = []
         carried_delta = 0
         for tfs in sliced_template:
             if delta_stack:
@@ -803,10 +834,10 @@ class JinjaTemplater(PythonTemplater):
 
     @staticmethod
     def _calculate_variant_score(
-        raw_sliced: List[RawFileSlice],
-        sliced_file: List[TemplatedFileSlice],
-        uncovered_slices: Set[int],
-        original_source_slices: Dict[int, slice],
+        raw_sliced: list[RawFileSlice],
+        sliced_file: list[TemplatedFileSlice],
+        uncovered_slices: set[int],
+        original_source_slices: dict[int, slice],
     ) -> int:
         """Compute a score for the variant based from size of covered slices.
 
@@ -839,9 +870,9 @@ class JinjaTemplater(PythonTemplater):
         self,
         in_str: str,
         render_func: Callable[[str], str],
-        uncovered_slices: Set[int],
-        append_to_templated="",
-    ):
+        uncovered_slices: set[int],
+        append_to_templated: str = "",
+    ) -> Iterator[tuple[list[RawFileSlice], list[TemplatedFileSlice], str]]:
         """Address uncovered slices by tweaking the template to hit them.
 
         Args:
@@ -860,7 +891,7 @@ class JinjaTemplater(PythonTemplater):
 
         max_variants_generated = 10
         max_variants_returned = 5
-        variants: Dict[str, Tuple[int, JinjaTrace, Dict[int, int]]] = {}
+        variants: dict[str, tuple[int, JinjaTrace, dict[int, int]]] = {}
 
         # Create a mapping of the original source slices before modification so
         # we can adjust the positions post-modification.
@@ -876,7 +907,7 @@ class JinjaTemplater(PythonTemplater):
             # `length_deltas` is to keep track of the length changes associated
             # with the changes we're making so we can correct the positions in
             # the resulting template.
-            length_deltas: Dict[int, int] = {}
+            length_deltas: dict[int, int] = {}
             # Find a path that takes us to 'uncovered_slice'.
             choices = tracer_probe.move_to_slice(uncovered_slice, 0)
             for branch, options in choices.items():
@@ -936,7 +967,7 @@ class JinjaTemplater(PythonTemplater):
                     variants[variant_raw_str] = (score, trace, length_deltas)
 
         # Return the top-scoring variants.
-        sorted_variants: List[Tuple[int, JinjaTrace, Dict[int, int]]] = sorted(
+        sorted_variants: list[tuple[int, JinjaTrace, dict[int, int]]] = sorted(
             variants.values(), key=lambda v: v[0], reverse=True
         )
         for _, trace, deltas in sorted_variants[:max_variants_returned]:
@@ -955,8 +986,13 @@ class JinjaTemplater(PythonTemplater):
 
     @large_file_check
     def process_with_variants(
-        self, *, in_str: str, fname: str, config=None, formatter=None
-    ) -> Iterator[Tuple[Optional[TemplatedFile], List[SQLTemplaterError]]]:
+        self,
+        *,
+        in_str: str,
+        fname: str,
+        config: Optional[FluffConfig] = None,
+        formatter: Optional[FormatterInterface] = None,
+    ) -> Iterator[tuple[TemplatedFile, list[SQLTemplaterError]]]:
         """Process a string and return one or more variant renderings.
 
         Note that the arguments are enforced as keywords
@@ -981,9 +1017,6 @@ class JinjaTemplater(PythonTemplater):
         )
         yield templated_file, violations
 
-        if not templated_file:
-            return  # pragma: no cover
-
         # Find uncovered code (if any), tweak the template to hit that code.
         # First, identify the literals which _are_ covered.
         covered_literal_positions = {
@@ -1024,6 +1057,27 @@ class JinjaTemplater(PythonTemplater):
                 violations,
             )
 
+    @staticmethod
+    def _exclude_macros(macro_path: str, exclude_macros_path: list[str]) -> bool:
+        """Determines if a macro is within the exclude macros path.
+
+        These macros will be ignored and not loaded into context
+
+        Args:
+            macro_path (str): Str of the path to the macro
+            exclude_macros_path (list[str]): Str of the path to the macros to exclude
+
+        Returns:
+            bool: True if the macro should be excluded
+        """
+        for exclude_path in exclude_macros_path:
+            macro_path_normalized = os.path.normpath(os.path.abspath(macro_path))
+            exclude_path_normalized = os.path.normpath(exclude_path)
+            if exclude_path_normalized in macro_path_normalized:
+                templater_logger.debug("Skipping this macro file: %s", macro_path)
+                return True
+        return False
+
 
 class DummyUndefined(jinja2.Undefined):
     """Acts as a dummy value to try and avoid template failures.
@@ -1039,7 +1093,7 @@ class DummyUndefined(jinja2.Undefined):
     # https://jinja.palletsprojects.com/en/3.0.x/sandbox/#jinja2.sandbox.SandboxedEnvironment.is_safe_callable
     alters_data = False
 
-    def __init__(self, name) -> None:
+    def __init__(self, name: str) -> None:
         super().__init__()
         self.name = name
 
@@ -1047,7 +1101,7 @@ class DummyUndefined(jinja2.Undefined):
         return self.name.replace(".", "_")
 
     @classmethod
-    def create(cls, name) -> "DummyUndefined":
+    def create(cls, name: str) -> "DummyUndefined":
         """Factory method.
 
         When ignoring=templating is configured, use 'name' as the value for
@@ -1062,7 +1116,7 @@ class DummyUndefined(jinja2.Undefined):
         result = DummyUndefined(name)
         return result
 
-    def __getattr__(self, item):
+    def __getattr__(self, item: str) -> "DummyUndefined":
         """Intercept any calls to undefined attributes.
 
         Args:
@@ -1076,7 +1130,7 @@ class DummyUndefined(jinja2.Undefined):
     # Implement the most common magic methods. This helps avoid
     # templating errors for undefined variables.
     # https://www.tutorialsteacher.com/python/magic-methods-in-python
-    def _self_impl(self, *args, **kwargs) -> "DummyUndefined":
+    def _self_impl(self, *args: Any, **kwargs: Any) -> "DummyUndefined":
         """Return an instance of the class itself.
 
         Args:
@@ -1088,7 +1142,7 @@ class DummyUndefined(jinja2.Undefined):
         """
         return self
 
-    def _bool_impl(self, *args, **kwargs) -> bool:
+    def _bool_impl(self, *args: Any, **kwargs: Any) -> bool:
         """Return a boolean value.
 
         Args:
@@ -1100,32 +1154,23 @@ class DummyUndefined(jinja2.Undefined):
         """
         return True
 
-    __add__ = _self_impl
-    __sub__ = _self_impl
-    __mul__ = _self_impl
-    __floordiv__ = _self_impl
-    __truediv__ = _self_impl
-    __mod__ = _self_impl
-    __pow__ = _self_impl
-    __pos__ = _self_impl
-    __neg__ = _self_impl
-    __lshift__ = _self_impl
-    __rshift__ = _self_impl
-    __getitem__ = _self_impl
+    # We're intentionally changing the behaviour here compared to the base
+    # class of Undefined. That means we're going to silence the `assignment`
+    # mypy warnings. Operations on an undefined result in another undefined.
+    __add__ = __sub__ = __mul__ = _self_impl  # type: ignore[assignment]
+    __floordiv__ = __truediv__ = _self_impl  # type: ignore[assignment]
+    __mod__ = __pow__ = _self_impl  # type: ignore[assignment]
+    __pos__ = __neg__ = _self_impl  # type: ignore[assignment]
+    __lshift__ = __rshift__ = _self_impl
+    __getitem__ = _self_impl  # type: ignore[assignment]
     __invert__ = _self_impl
-    __call__ = _self_impl
-    __and__ = _bool_impl
-    __or__ = _bool_impl
-    __xor__ = _bool_impl
-    __bool__ = _bool_impl
-    __lt__ = _bool_impl
-    __le__ = _bool_impl
-    __eq__ = _bool_impl
-    __ne__ = _bool_impl
-    __ge__ = _bool_impl
-    __gt__ = _bool_impl
+    __call__ = _self_impl  # type: ignore[assignment]
+    # Boolean operations on an undefined are handled separately.
+    __and__ = __or__ = __xor__ = __bool__ = _bool_impl
+    __lt__ = __le__ = __ge__ = __gt__ = _bool_impl  # type: ignore[assignment]
+    __eq__ = __ne__ = _bool_impl
 
-    def __hash__(self) -> int:  # pragma: no cov
+    def __hash__(self) -> int:  # pragma: no cover
         """Return a constant hash value.
 
         Returns:
@@ -1134,7 +1179,7 @@ class DummyUndefined(jinja2.Undefined):
         # This is called by the "in" operator, among other things.
         return 0
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator["DummyUndefined"]:
         """Return an iterator that contains only the instance of the class itself.
 
         Returns:
@@ -1148,7 +1193,7 @@ class DBTTestExtension(Extension):
 
     tags = {"test"}
 
-    def parse(self, parser) -> jinja2.nodes.Macro:
+    def parse(self, parser: jinja2.parser.Parser) -> jinja2.nodes.Macro:
         """Parses out the contents of the test tag."""
         node = jinja2.nodes.Macro(lineno=next(parser.stream).lineno)
         test_name = parser.parse_assign_target(name_only=True).name
