@@ -255,9 +255,12 @@ UNRESERVED_KEYWORDS = [
     "ABORT_AFTER_WAIT",
     "ABORT",
     "ABSENT",
+    "ABSOLUTE",
+    "ACCELERATED_DATABASE_RECOVERY",
     "ACTION",
     "AFTER",
     "ALGORITHM",
+    "ALLOW_CONNECTIONS",
     "ALLOW_ENCRYPTED_VALUE_MODIFICATIONS",
     "ALLOW_PAGE_LOCKS",
     "ALLOW_ROW_LOCKS",
@@ -276,8 +279,16 @@ UNRESERVED_KEYWORDS = [
     "ASYMMETRIC",
     "AT",
     "ATOMIC",
+    "ATTACH",
+    "ATTACH_REBUILD_LOG",
+    "AUTO_CLOSE",
+    "AUTO_CREATE_STATISTICS",
     "AUTO_CREATE_TABLE",
+    "AUTO_SHRINK",
+    "AUTO_UPDATE_STATISTICS",
+    "AUTO_UPDATE_STATISTICS_ASYNC",
     "AUTO",
+    "BACKUP_STORAGE_REDUNDANCY",
     "BEFORE",  # *future*
     "BERNOULLI",
     "BINARY",
@@ -288,6 +299,7 @@ UNRESERVED_KEYWORDS = [
     "CALLED",
     "CALLER",
     "CAST",
+    "CATALOG_COLLATION",
     "CATCH",
     "CERTIFICATE",
     "CHANGE_TRACKING",
@@ -298,6 +310,7 @@ UNRESERVED_KEYWORDS = [
     "COLUMNSTORE_ARCHIVE",
     "COLUMNSTORE",
     "COMMITTED",
+    "COMPATIBILITY_LEVEL",
     "COMPRESS_ALL_ROW_GROUPS",
     "COMPRESSION_DELAY",
     "COMPRESSION",
@@ -306,6 +319,7 @@ UNRESERVED_KEYWORDS = [
     "CONNECTION",
     "CONNECTION_OPTIONS",
     "CONTAINED",
+    "CONTAINMENT",
     "CONTEXT_INFO",
     "CONTINUE",
     "CONTROL",
@@ -318,6 +332,7 @@ UNRESERVED_KEYWORDS = [
     "DATA_DELETION",
     "DATA_SOURCE",
     "DATA",
+    "DATABASE_DEFAULT",
     "DATAFILETYPE",
     "DATASOURCE",
     "DATE_FORMAT",
@@ -326,8 +341,10 @@ UNRESERVED_KEYWORDS = [
     "DATEFORMAT",
     "DAY",
     "DAYS",
+    "DB_CHAINING",
     "DEADLOCK_PRIORITY",
     "DEFAULT_DATABASE",
+    "DEFAULT_FULLTEXT_LANGUAGE",
     "DEFAULT_LANGUAGE",
     "DEFAULT_SCHEMA",
     "DELAY",
@@ -336,6 +353,7 @@ UNRESERVED_KEYWORDS = [
     "DELTA",
     "DENSE_RANK",
     "DETERMINISTIC",
+    "DIRECTORY_NAME",
     "DISABLE",
     "DISK",  # listed as reserved but functionally unreserved
     "DISTRIBUTION",  # Azure Synapse Analytics specific
@@ -343,12 +361,16 @@ UNRESERVED_KEYWORDS = [
     "DUMP",  # listed as reserved but functionally unreserved
     "DURABILITY",
     "EDGE",
+    "EDITION",
+    "ELASTIC_POOL",
     "ELEMENT",  # *future*
     "ELEMENTS",
+    "ENABLE_BROKER",
     "ENCODING",
     "ENCRYPTED",
     "ENCRYPTION_TYPE",
     "ENCRYPTION",
+    "ERROR_BROKER_CONVERSATIONS",
     "ERRORFILE_CREDENTIAL",
     "ERRORFILE_DATA_SOURCE",
     "ERRORFILE",
@@ -356,6 +378,7 @@ UNRESERVED_KEYWORDS = [
     "EXPLAIN",  # Azure Synapse Analytics specific
     "EXPLICIT",
     "EXTERNALPUSHDOWN",
+    "FAILOVER",
     "FAST",
     "FIELD_TERMINATOR",
     "FIELDQUOTE",
@@ -363,9 +386,10 @@ UNRESERVED_KEYWORDS = [
     "FILE_FORMAT",
     "FILE_TYPE",
     "FILEGROUP",
+    "FILEGROWTH",
+    "FILENAME",
     "FILESTREAM_ON",
     "FILESTREAM",
-    "FILESTREAM",
     "FILETABLE_COLLATE_FILENAME",
     "FILETABLE_DIRECTORY",
     "FILETABLE_FULLPATH_UNIQUE_CONSTRAINT_NAME",
@@ -383,6 +407,7 @@ UNRESERVED_KEYWORDS = [
     "FOLLOWING",
     "FORCE",
     "FORCED",
+    "FORCE_FAILOVER_ALLOW_DATA_LOSS",
     "FORCEPLAN",
     "FORCESCAN",
     "FORCESEEK",
@@ -392,7 +417,10 @@ UNRESERVED_KEYWORDS = [
     "FORMATFILE_DATA_SOURCE",
     "FORMATFILE",
     "FULLTEXT",
+    "GB",
     "GENERATED",
+    "GEO",
+    "GEOZONE",
     "HASH",
     "HEAP",  # Azure Synapse Analytics specific
     "HIDDEN",
@@ -410,9 +438,11 @@ UNRESERVED_KEYWORDS = [
     "INCLUDE_NULL_VALUES",
     "INCLUDE",
     "INCREMENT",
+    "INCREMENTAL",
     "INFINITE",
     "INLINE",
     "INPUT",
+    "INSENSITIVE",
     "INSTEAD",
     "INTERVAL",
     "IO",
@@ -420,6 +450,7 @@ UNRESERVED_KEYWORDS = [
     "JSON",
     "JSON_ARRAY",
     "JSON_OBJECT",
+    "KB",
     "KEEP",
     "KEEPDEFAULTS",
     "KEEPFIXED",
@@ -443,6 +474,7 @@ UNRESERVED_KEYWORDS = [
     "LOOP",
     "LOW",
     "MANUAL",
+    "MANUAL_CUTOVER",
     "MASKED",
     "MASTER",
     "MATCHED",
@@ -452,16 +484,23 @@ UNRESERVED_KEYWORDS = [
     "MAXDOP",
     "MAXERRORS",
     "MAXRECURSION",
+    "MAXSIZE",
     "MAXVALUE",
+    "MB",
     "MEMORY_OPTIMIZED",
+    "MEMORY_OPTIMIZED_DATA",
     "MIGRATION_STATE",
     "MIN_GRANT_PERCENT",
     "MINUTES",
     "MINVALUE",
+    "MODIFY",
     "MONTH",
     "MONTHS",
     "MUST_CHANGE",
     "NAME",
+    "NESTED_TRIGGERS",
+    "NEWNAME",
+    "NEW_BROKER",
     "NEXT",
     "NO_PERFORMANCE_SPOOL",
     "NO",
@@ -471,6 +510,7 @@ UNRESERVED_KEYWORDS = [
     "NOEXPAND",
     "NOLOCK",
     "NONE",
+    "NON_TRANSACTED_ACCESS",
     "NORMAL",
     "NOWAIT",
     "NTILE",
@@ -511,8 +551,11 @@ UNRESERVED_KEYWORDS = [
     "PERCENTAGE",
     "PERCENTILE_CONT",
     "PERCENTILE_DISC",
+    "PERFORM_CUTOVER",
     "PERIOD",
     "PERSISTED",
+    "PERSISTENT_LOG_BUFFER",
+    "PERSISTENT_VERSION_STORE_FILEGROUP",
     "POLICY",
     "POPULATION",
     "PRECEDING",
@@ -536,6 +579,7 @@ UNRESERVED_KEYWORDS = [
     "READCOMMITTEDLOCK",
     "READONLY",
     "READPAST",
+    "READ_WRITE",
     "READUNCOMMITTED",
     "REBUILD",
     "RECEIVE",
@@ -555,8 +599,10 @@ UNRESERVED_KEYWORDS = [
     "REJECT_TYPE",
     "REJECT_VALUE",
     "REJECTED_ROW_LOCATION",
+    "RELATIVE",
     "REMOTE_DATA_ARCHIVE",
     "REMOTE_PROC_TRANSACTIONS",
+    "REMOVE",
     "RENAME",  # Azure Synapse Analytics specific
     "REORGANIZE",
     "REPEATABLE",
@@ -567,6 +613,7 @@ UNRESERVED_KEYWORDS = [
     "RESULT",
     "RESULT_SET_CACHING",  # Azure Synapse Analytics specific
     "RESUMABLE",
+    "RESTRICTED_USER",
     "RESUME",
     "RETENTION_PERIOD",
     "RETURNS",
@@ -590,6 +637,7 @@ UNRESERVED_KEYWORDS = [
     "SCHEME",
     "SCOPED",
     "SEARCH",
+    "SECONDARY",
     "SECRET",
     "SECURITY",
     "SECURITYAUDIT",  # listed as reserved but functionally unreserved
@@ -601,6 +649,7 @@ UNRESERVED_KEYWORDS = [
     "SERIALIZABLE",
     "SERVER",
     "SERVICE",
+    "SERVICE_OBJECTIVE",
     "SETERROR",
     "SETS",
     "SETVAR",  # sqlcmd command
@@ -611,6 +660,7 @@ UNRESERVED_KEYWORDS = [
     "SINGLE_BLOB",
     "SINGLE_CLOB",
     "SINGLE_NCLOB",
+    "SIZE",
     "SNAPSHOT",
     "SORT_IN_TEMPDB",
     "SOURCE",
@@ -630,6 +680,7 @@ UNRESERVED_KEYWORDS = [
     "SYSTEM_TIME",
     "SYSTEM_VERSIONING",
     "SYSTEM",
+    "TB",
     "TABLOCK",
     "TABLOCKX",
     "TAKE",
@@ -641,13 +692,17 @@ UNRESERVED_KEYWORDS = [
     "TIMESTAMP",
     "TRANSACTION_ID_COLUMN_NAME",
     "TRANSACTION_ID",
+    "TRANSFORM_NOISE_WORDS",
     "TRUNCATE_TARGET",  # Azure Synapse Analytics specific
+    "TRUSTWORTHY",
     "TRY",
     "TYPE",
+    "TWO_DIGIT_YEAR_CUTOFF",
     "UNBOUNDED",
     "UNCOMMITTED",
     "UNDEFINED",
     "UNKNOWN",
+    "UNLIMITED",
     "UPDLOCK",
     "USE_TYPE_DEFAULT",
     "USED",
